@@ -1,13 +1,13 @@
-# $Id: Cache.pm,v 1.30 2001/12/30 16:09:56 matt Exp $
+# $Id: Cache.pm,v 1.5.2.1 2002/06/06 20:00:19 matts Exp $
 
 package Apache::AxKit::Cache;
 use strict;
 
 use Apache;
-use Apache::Constants qw(OK DECLINED);
+use Apache::Constants qw(OK DECLINED SERVER_ERROR);
 use Apache::AxKit::Exception;
 use Digest::MD5 ();
-use Compress::Zlib;
+use Compress::Zlib qw(gzopen);
 use Fcntl qw(:flock O_RDWR O_WRONLY O_CREAT O_RDONLY);
 
 # use vars qw/$COUNT/;
@@ -17,11 +17,12 @@ sub new {
     my ($r, $xmlfile, @extras) = @_;
     
     my $gzip = 0;
-    if ($xmlfile =~ /.gzip/) {
+    if ($xmlfile =~ /\.gzip/) {
         $gzip++;
 #        @extras = grep(!/gzip/, @extras);
     }
     
+    local $^W; # suppress "Use of uninitialized value" warnings
     my $key = Digest::MD5->new->add(
             join(':', 
                 $r->get_server_name,
@@ -92,6 +93,9 @@ sub _get_stats {
     return if $self->{mtime};
     my @stats = stat($self->{file});
     my $exists = -e _ && -r _;
+    if ($exists and $self->{gzip}) {
+        $exists = -e $self->{file} . '.gz' and -r _;
+    }
     $self->{file_exists} = $exists;
     $self->{mtime} = $stats[9];
 }
@@ -120,14 +124,10 @@ sub write {
     }
     
     if ($self->{gzip} && $AxKit::Cfg->GzipOutput) {
-        AxKit::Debug(3, 'Creating gzip output cache');
-        my $fh = Apache->gensym();
-        if (sysopen($fh, $self->{file}.'new.gz', O_RDWR|O_CREAT)) {
-            flock($fh, LOCK_EX);
-            seek($fh, 0, 0);
-            truncate($fh, 0);
-            print $fh ''.Compress::Zlib::memGzip($_[0]);
-            close $fh;
+        AxKit::Debug(3, "Creating gzip output cache: $self->{file}.gz");
+        if (my $gz = gzopen($self->{file}.'new.gz', "wb")) {
+            $gz->gzwrite($_[0]);
+            $gz->gzclose();
             rename($self->{file}.'new.gz', $self->{file}.'.gz')
                     || throw Apache::AxKit::Exception::IO( -text => "Couldn't rename gzipped cachefile: $!");
         }
@@ -199,7 +199,7 @@ sub get_type {
 
 sub deliver {
     my $self = shift;
-    return if $self->{no_cache};
+    return SERVER_ERROR if $self->{no_cache};
     my $r = $self->{apache};
 
     {
