@@ -1,4 +1,4 @@
-# $Id: HtmlDoc.pm,v 1.4.2.1 2003/07/28 22:51:19 matts Exp $
+# $Id: HtmlDoc.pm,v 1.2 2002/05/26 16:47:58 matts Exp $
 # Apache::AxKit::Language::HtmlDoc - xhtml->pdf renderer
 package Apache::AxKit::Language::HtmlDoc;
 
@@ -10,9 +10,8 @@ use Apache;
 use Apache::Constants qw(:common);
 use Apache::Request;
 use Apache::AxKit::Language;
-use Apache::AxKit::LibXMLSupport;
+use Apache::AxKit::Language::LibXSLT;
 use Apache::AxKit::Provider;
-use XML::LibXSLT;
 use IPC::Run qw(run);
 use Cwd;
 
@@ -26,9 +25,10 @@ sub handler {
     my ($r, $xml_provider, undef, $last_in_chain) = @_;
 
     my $parser = XML::LibXML->new();
-    local($XML::LibXML::match_cb, $XML::LibXML::open_cb,
-        $XML::LibXML::read_cb, $XML::LibXML::close_cb);
-    Apache::AxKit::LibXMLSupport->reset();
+    local $XML::LibXML::match_cb = \&Apache::AxKit::Language::LibXSLT::match_uri;
+    local $XML::LibXML::open_cb = \&Apache::AxKit::Language::LibXSLT::open_content_uri;
+    local $XML::LibXML::read_cb = \&Apache::AxKit::Language::LibXSLT::read_uri;
+    local $XML::LibXML::close_cb = \&Apache::AxKit::Language::LibXSLT::close_uri;
 
     my $dom;
     my $source_text;
@@ -68,9 +68,9 @@ EOX
     $input =~ s{ href="(?!/|.{0,5}:)}{ href="http://$host$path}g;
     AxKit::Debug(8, "About to shell out to htmldoc - hope you have it installed...");
     AxKit::Debug(10, $input);
-    run(['htmldoc','--quiet','--format','pdf13','--truetype','--size','a4','--color','--charset','8859-15','--webpage',$r->dir_config->get('AxHtmlDocOptions'),'-'],\$input,\$result);
+    my $rc = run(['htmldoc','--quiet','--format','pdf13','--truetype','--size','a4','--color','--charset','8859-15','--webpage',$r->dir_config->get('AxHtmlDocOptions'),'-'],\$input,\$result);
 
-    if (substr($result,0,5) ne '%PDF-') {
+    if (!$rc) {
         throw Apache::AxKit::Exception::Error(-text => 'htmldoc returned error: '.$result);
     }
 
