@@ -1,4 +1,4 @@
-# $Id: Sablot.pm,v 1.2 2002/01/30 18:17:21 darobin Exp $
+# $Id: Sablot.pm,v 1.4 2002/05/26 16:47:58 matts Exp $
 
 package Apache::AxKit::Language::Sablot;
 
@@ -61,8 +61,10 @@ sub handler {
     }
 
     print $xslt_processor->GetResultArg("result");
-    
+
     $xslt_processor->FreeResultArgs();
+
+    return Apache::Constants::OK;
 }
 
 END {
@@ -129,7 +131,7 @@ sub SHGetAll {
     
     my $uri = $scheme . $rest;
     
-    $uri =~ s/^axkit\///;
+    $uri =~ s/^sabaxkit\///;
     
     AxKit::Debug(8, "Sablot: Looking up URI: $uri\n");
     
@@ -151,7 +153,7 @@ BEGIN {
     package Apache::AxKit::Language::Sablot;
 
     $xslt_processor = XML::Sablotron->new();
-    $xslt_processor->SetBase("axkit:");
+    $xslt_processor->SetBase("sabaxkit:");
 
     $handler = Apache::AxKit::Language::Sablot::Handler->new();
     
