@@ -1,4 +1,4 @@
-# $Id: XPathScript.pm,v 1.1 2002/01/13 20:45:11 matts Exp $
+# $Id: XPathScript.pm,v 1.7 2002/05/31 19:22:24 matts Exp $
 
 package Apache::AxKit::Language::XPathScript;
 
@@ -89,9 +89,7 @@ sub handler {
         compile($package, $style_provider);
         $stash->{$style_key}{mtime} = get_mtime($class, $style_provider);
     }
-    
-    my $old_status = $r->status;
-    
+
     no strict 'refs';
     my $cv = \&{"$package\::handler"};
 
@@ -99,29 +97,26 @@ sub handler {
     my $t = {};
     local $Apache::AxKit::Language::XPathScript::trans = $t;
     local $Apache::AxKit::Language::XPathScript::style_provider = $style_provider;
-    
+
     AxKit::Debug(7, "Running XPathScript script\n");
     local $^W;
-    eval {
-        $cv->($r, $xpath, $t);
-    };
-    if ($@) {
-        AxKit::Debug(1, "XPathScript error: $@");
-        throw $@;
-    }
-    
-    if (!$r->pnotes('xml_string') && 
+    my $rc = Apache::Constants::OK;
+
+    # Execute the page
+    $rc = $cv->($r, $xpath, $t);
+
+    if (!$r->pnotes('xml_string') &&
         !$r->dir_config('XPSNoApplyTemplatesOnEmptyOutput')) { # no output? Try apply_templates
         print Apache::AxKit::Language::XPathScript::Toys::apply_templates();
     }
-    
+
 #    warn "Run\n";
 
     $Apache::AxKit::Language::XPathScript::xp = undef;
     $Apache::AxKit::Language::XPathScript::trans = undef;
     $Apache::AxKit::Language::XPathScript::style_provider = undef;
 #    warn "Returning $old_status\n";
-    return $r->status($old_status);
+    return $rc;
 }
 
 sub get_source_tree {
@@ -130,19 +125,24 @@ sub get_source_tree {
     AxKit::Debug(7, "XPathScript: reparsing file");
     eval {
         my $fh = $provider->get_fh();
-        # warn("parsing FH $fh with parser $parser\n");
+# warn("parsing FH $fh with parser $parser\n");
         local $/;
         my $contents = <$fh>;
-        # warn("FH contains: $contents\n");
+# warn("FH contains: $contents\n");
         $source_tree = $parser->parse($contents);
-        # warn("Parse completed\n");
+# warn("Parse completed\n");
         close($fh);
+# warn("closed filehandle\n");
     };
     if ($@) {
-        # warn("parse_fh failed\n");
-        $source_tree = $parser->parse(${ $provider->get_strref() });
+# warn("parse_fh failed\n");
+        my $str = $provider->get_strref();
+# warn("Got str\n");
+        $source_tree = $parser->parse($$str);
+# warn("Got source tree\n");
     }
-
+    
+    AxKit::Debug(7, "XPathScript: Returning parsed source tree");
     # warn("get_source_tree = $source_tree\n");
     return $source_tree;
 }
@@ -150,13 +150,15 @@ sub get_source_tree {
 sub check_inc_mtime {
     my ($mtime, $provider, $includes) = @_;
     
+    my $apache = $provider->apache_request;
+    
     for my $inc (@$includes) {
 #        warn "Checking mtime for $inc\n";
-        my $inc_provider = Apache::AxKit::Provider->new(
-                AxKit::Apache->request, 
-                uri => $inc,
-                rel => $provider,
-                );
+        my $sub = $apache->lookup_uri($inc);
+        local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
+        
+        my $inc_provider = Apache::AxKit::Provider->new_style_provider($sub);
+        
         if ($inc_provider->has_changed($mtime)) {
 #            warn "$inc newer (" . $inc_provider->mtime() . ") than last compile ($mtime) causing recompile\n";
             return;
@@ -268,11 +270,13 @@ sub compile {
             'my ($r, $xp, $t) = @_;',
             "\n#line 1 " . $provider->key() . "\n",
             $script,
+            ";\n",
+            'return Apache::Constants::OK;',
             "\n}",
             );
-    
+
     local $^W;
-    
+
     AxKit::Debug(10, "Compiling script:\n$eval\n");
     eval $eval;
     if ($@) {
@@ -287,14 +291,14 @@ sub include_file {
     # return if already included
     my $key = $provider->key();
     return '' if grep {$_ eq $filename} @{$stash->{$key}{includes}};
-    
+
     push @{$stash->{$key}{includes}}, $filename;
     
-    my $inc_provider = Apache::AxKit::Provider->new(
-            AxKit::Apache->request,
-            uri => $filename,
-            rel => $provider,
-            );
+    my $apache = $provider->apache_request;
+    my $sub = $apache->lookup_uri($filename);
+    local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
+    
+    my $inc_provider = Apache::AxKit::Provider->new_style_provider($sub);
     
     return extract($inc_provider, $script_output);
 }
@@ -317,7 +321,10 @@ sub XML::XPath::Function::document {
     my $results = XML::XPath::NodeSet->new();
     my $uri = $params[0];
     my $newdoc;
-    if ($uri =~ /^\w\w+:/) { # assume it's scheme://foo uri
+    if ($uri =~ /^axkit:/) {
+        $newdoc = $parser->parse( AxKit::get_axkit_uri($uri) );
+    }
+    elsif ($uri =~ /^\w\w+:/) { # assume it's scheme://foo uri
         eval {
             # warn "Trying to parse $params[0]\n";
             $newdoc = $parser->parse(
@@ -337,15 +344,23 @@ sub XML::XPath::Function::document {
         }
     }
     else {
-        # warn("Parsing local: $uri\n");
-        my $provider = Apache::AxKit::Provider->new(
-                    AxKit::Apache->request, 
-                    uri => $uri,
-                );
+        AxKit::Debug(3, "Parsing local: $uri\n");
+        
+        # create a subrequest, so we get the right AxKit::Cfg for the URI
+        my $apache = AxKit::Apache->request;
+        my $sub = $apache->lookup_uri($uri);
+        local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
+        
+        my $provider = Apache::AxKit::Provider->new_content_provider($sub);
+        
         $newdoc = get_source_tree($provider, $parser);
+        undef $provider;
+        undef $apache;
+        undef $sub;
     }
 
     $results->push($newdoc) if $newdoc;
+    #AxKit::Debug(8, "XPathScript: document() returning");
     return $results;
 }
 
@@ -364,11 +379,16 @@ sub get_mtime {
         return 0;
     }
 
+    my $apache = $provider->apache_request;
+    
     for my $inc (@{$stash->{$filename}{includes}}) {
-        my $inc_provider = Apache::AxKit::Provider->new(
-                AxKit::Apache->request, 
-                uri => $inc,
-                rel => $provider,
+        
+        my $sub = $apache->lookup_uri($inc);
+        local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
+        
+        my $inc_provider = Apache::AxKit::Provider->new_style_provider(
+                $sub, 
+                # uri => $inc,
                 );
         
 #        warn "Checking mtime of $inc\n";
@@ -469,7 +489,7 @@ sub get_package_name {
     sub matches {
         $Apache::AxKit::Language::XPathScript::xp->matches(@_);
     }
-    
+
     sub set_namespace {
         eval {
             $Apache::AxKit::Language::XPathScript::xp->set_namespace(@_);
@@ -529,7 +549,7 @@ sub get_package_name {
         
         return $retval;
     }
-    
+
     sub translate_node {
         my $node = shift;
         
@@ -589,7 +609,7 @@ sub get_package_name {
                 return empty_tag($node);
             }
         }
-        
+
         local $^W;
         
         my $dokids = 1;
@@ -620,7 +640,7 @@ sub get_package_name {
 #            warn "Here with $result\n";
             
 
-            if ($result =~ /\D/) {
+            if ($result !~ /^-?\d+$/) {
                 $dokids = 0;
                 $search = $result;
             }
@@ -657,7 +677,7 @@ sub get_package_name {
         my $post = interpolate($node, $trans->{postchildren}, $testcode_output) .
                 ($trans->{showtag} ? end_tag($node) : '') .
                 interpolate($node, $trans->{post}, $testcode_output);
-        
+
         if ($dokids) {
             my $middle = '';
             for my $kid ($node->getChildNodes()) {
