@@ -1,4 +1,4 @@
-# $Id: XSP.pm,v 1.8 2002/03/10 16:16:59 matts Exp $
+# $Id: XSP.pm,v 1.13.2.2 2002/06/08 08:41:53 matts Exp $
 
 package Apache::AxKit::Language::XSP;
 
@@ -308,6 +308,7 @@ sub start_document {
     foreach my $ns (keys %Apache::AxKit::Language::XSP::tag_lib) {
         my $pkg = $Apache::AxKit::Language::XSP::tag_lib{$ns};
         my $sub;
+        local $AxKit::XSP::TaglibPkg = $pkg;
         if (($sub = $pkg->can("start_document")) && ($sub != \&start_document)) {
             $e->{XSP_Script} .= $sub->($e);
         }
@@ -323,6 +324,7 @@ sub end_document {
     foreach my $ns (keys %Apache::AxKit::Language::XSP::tag_lib) {
         my $pkg = $Apache::AxKit::Language::XSP::tag_lib{$ns};
         my $sub;
+        local $AxKit::XSP::TaglibPkg = $pkg;
         if (($sub = $pkg->can("end_document")) && ($sub != \&end_document)) {
             $e->{XSP_Script} .= $sub->($e);
         }
@@ -389,6 +391,7 @@ sub start_element {
         $e->manage_text(1); # set default for xsp tags
         my $pkg = $Apache::AxKit::Language::XSP::tag_lib{ $ns };
         my $sub;
+        local $AxKit::XSP::TaglibPkg = $pkg;
         if (($sub = $pkg->can("start_element")) && ($sub != \&start_element)) {
             $e->{XSP_Script} .= $sub->($e, $element);
         }
@@ -417,6 +420,7 @@ sub end_element {
         my $tag = $element->{Name};
         my $pkg = $Apache::AxKit::Language::XSP::tag_lib{ $ns };
         my $sub;
+        local $AxKit::XSP::TaglibPkg = $pkg;
         if (($sub = $pkg->can("end_element")) && ($sub != \&end_element)) {
             $e->{XSP_Script} .= $sub->($e, $element);
         }
@@ -444,6 +448,7 @@ sub characters {
     else {
         my $pkg = $Apache::AxKit::Language::XSP::tag_lib{ $ns };
         my $sub;
+        local $AxKit::XSP::TaglibPkg = $pkg;
         if (($sub = $pkg->can("characters")) && ($sub != \&characters)) {
             $e->{XSP_Script} .= $sub->($e, $text);
         }
@@ -468,6 +473,7 @@ sub comment {
 #        local $^W;
         my $pkg = $Apache::AxKit::Language::XSP::tag_lib{ $ns };
         my $sub;
+        local $AxKit::XSP::TaglibPkg = $pkg;
         if (($sub = $pkg->can("comment")) && ($sub != \&comment)) {
             $e->{XSP_Script} .= $sub->($e, $comment);
         }
@@ -492,6 +498,7 @@ sub processing_instruction {
 #        local $^W;
         my $pkg = $Apache::AxKit::Language::XSP::tag_lib{ $ns };
         my $sub;
+        local $AxKit::XSP::TaglibPkg = $pkg;
         if (($sub = $pkg->can("processing_instruction")) && ($sub != \&processing_instruction)) {
             $e->{XSP_Script} .= $sub->($e, $pi);
         }
@@ -838,7 +845,7 @@ sub end_element {
     my ($e, $element) = @_;
     
     $e->{Current_NS} = pop @{ $e->{NS_Stack} };
-    
+
     return '$parent = $parent->getParentNode;' . "\n";
 }
 
@@ -888,6 +895,10 @@ sub parse {
     my $doc;
 
     my $parser = XML::LibXML->new();
+    local $XML::LibXML::match_cb = \&match_uri;
+    local $XML::LibXML::open_cb = \&open_content_uri;
+    local $XML::LibXML::read_cb = \&read_uri;
+    local $XML::LibXML::close_cb = \&close_uri;
     $parser->expand_entities(1);
     
     if (ref($thing)) {
@@ -910,6 +921,45 @@ sub parse {
     $self->{Handler}->end_document($document);
 }
 
+sub match_uri {
+    my $uri = shift;
+    AxKit::Debug(8, "XSP match_uri: $uri");
+    return 1 if $uri =~ /^axkit:/;
+    return $uri !~ /^\w+:/; # only handle URI's without a scheme
+}
+
+sub open_content_uri {
+    my $uri = shift || './';
+    AxKit::Debug(8, "XSP open_content_uri: $uri");
+    
+    if ($uri =~ /^axkit:/) {
+        return AxKit::get_axkit_uri($uri);
+    }
+    
+    # create a subrequest, so we get the right AxKit::Cfg for the URI
+    my $apache = AxKit::Apache->request;
+    my $sub = $apache->lookup_uri($uri);
+    local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($sub);
+    
+    my $provider = Apache::AxKit::Provider->new_content_provider($sub);
+    
+    AxKit::add_depends($provider->key());
+    my $str = $provider->get_strref;
+    
+    undef $provider;
+    undef $apache;
+    undef $sub;
+    
+    return $$str;
+}
+
+sub close_uri {
+}
+
+sub read_uri {
+    return substr($_[0], 0, $_[1], "");
+}
+
 sub process_node {
     my ($handler, $node, $encoding) = @_;
 
