@@ -1,9 +1,9 @@
-# $Id: QueryString.pm,v 1.6 2000/12/06 14:11:54 matt Exp $
+# $Id: QueryString.pm,v 1.2 2002/02/01 14:45:07 matts Exp $
 
 package Apache::AxKit::StyleChooser::QueryString;
 
 use strict;
-use Apache::Constants;
+use Apache::Constants qw(OK);
 
 sub handler {
 	my $r = shift;
@@ -12,7 +12,7 @@ sub handler {
 	if ($in{style}) {
 		$r->notes('preferred_style', $in{style});
 	}
-	return DECLINED;
+	return OK;
 }
 
 1;
@@ -22,6 +22,11 @@ __END__
 
 Apache::AxKit::StyleChooser::QueryString - Choose stylesheet using querystring
 
+=head1 SYNOPSIS
+
+	PerlHandler Apache::AxKit::StyleChooser::QueryString \
+			AxKit
+
 =head1 DESCRIPTION
 
 This module lets you pick a stylesheet based on the querystring. To use
