@@ -1,4 +1,4 @@
-# $Id: AxKit.pm,v 1.3 2002/03/15 13:37:52 matts Exp $
+# $Id: AxKit.pm,v 1.18.2.1 2002/06/08 12:03:10 matts Exp $
 
 package AxKit;
 use strict;
@@ -17,11 +17,13 @@ use Apache::AxKit::Provider::File;
 use Apache::AxKit::Provider::Scalar;
 use Apache::AxKit::CharsetConv;
 use File::Basename ();
+use Compress::Zlib ();
+use Fcntl;
 
 Apache::AxKit::CharsetConv::raise_error(1);
 
 BEGIN {
-    $VERSION = "1.51";
+    $VERSION = "1.6";
     if ($ENV{MOD_PERL}) {
         $AxKit::ServerString = "AxKit/$VERSION";
         @AxKit::ISA = qw(DynaLoader);
@@ -91,7 +93,7 @@ sub get_output_transformer {
             return map { $map->convert( $_ ) } ($outputfunc->(@_));
         };
     }
-    
+
     foreach my $AxOutputTransformer ( $AxKit::Cfg->OutputTransformers() ) {
         $actually_transform = 1;
         my $outputfunc = $func;
@@ -135,8 +137,13 @@ sub get_depends {
 
 sub fast_handler {
     my $r = shift;
-    
+
     local $SIG{__DIE__} = sub { AxKit::prep_exception(@_)->throw };
+
+    # use Carp ();
+    # local $SIG{'USR2'} = sub { 
+    #     Carp::confess("caught SIGUSR2!");
+    # };
     
     $AxKit::Cfg = Apache::AxKit::ConfigReader->new($r);
 
@@ -147,7 +154,9 @@ sub fast_handler {
 
     $Error::Debug = 1 if (($AxKit::Cfg->DebugLevel() > 3) || $AxKit::Cfg->StackTrace);
 
-    AxKit::Debug(1, "handler called for " . $r->uri);
+    AxKit::Debug(1, "fast handler called for " . $r->uri);
+    
+    local $AxKit::FastHandler = 1;
 
     my $plugin_ret = AxKit::run_plugins($r);
     if ($plugin_ret != OK) {
@@ -155,10 +164,10 @@ sub fast_handler {
         return $plugin_ret;
     }
 
-    my $provider = Apache::AxKit::Provider->new($r);
+    my $provider = Apache::AxKit::Provider->new_content_provider($r);
 
     return $provider->decline(reason => "passthru set")
-            if $r->notes('axkit_passthru');
+            if ($r->notes('axkit_passthru') && $r->dir_config('AxFastPassthru'));
 
     return main_handler($r, $provider);
 }
@@ -180,10 +189,13 @@ sub handler {
 #     ##############################
 
     local $SIG{__DIE__} = sub { AxKit::prep_exception(@_)->throw };
+
+    # use Carp ();
+    # local $SIG{'USR2'} = sub { 
+    #     Carp::confess("caught SIGUSR2!");
+    # };
     
     local $AxKit::Cfg;
-    local $AxKit::Cache;
-    local $AxKit::HeadersSent;
     local $AxKit::DebugLevel;
     local $Error::Debug;
 
@@ -194,125 +206,62 @@ sub handler {
         $AxKit::T0 = [Time::HiRes::gettimeofday()] if $AxKit::Cfg->DebugTime;
     }
 
-    $AxKit::DebugLevel = $AxKit::Cfg->DebugLevel();
-    $Error::Debug = 1 if (($AxKit::DebugLevel > 3) || $AxKit::Cfg->StackTrace);
+    $Error::Debug = 1 if (($AxKit::Cfg->DebugLevel() > 3) || $AxKit::Cfg->StackTrace);
 
     AxKit::Debug(1, "handler called for " . $r->uri);
 
+    local $AxKit::FastHandler = 0;
+
     my $plugin_ret = AxKit::run_plugins($r);
     if ($plugin_ret != OK) {
         AxKit::Debug(2, "Plugin returned non-OK value");
         return $plugin_ret;
     }
 
-    my $provider = Apache::AxKit::Provider->new($r);
+    my $provider = Apache::AxKit::Provider->new_content_provider($r);
 
     return $provider->decline(reason => "passthru set")
-            if $r->notes('axkit_passthru');
+            if ($r->notes('axkit_passthru') && $r->dir_config('AxFastPassthru'));
 
-    # Do we process this URL?
-    AxKit::Debug(2, "checking if we process this resource");
-    if (!$provider->process()) {
-        return $provider->decline();
-    }
-    
     return main_handler($r, $provider);
 }
 
 sub main_handler {
     my ($r, $provider) = @_;
+
+    # Do we process this URL?
+    # (moved down here from slow_handler because of AxHandleDirs)
+    AxKit::Debug(2, "checking if we process this resource");
+    if (!$provider->process()) {
+        return $provider->decline();
+    }
+
+    if ($r->notes('axkit_passthru')) {
+        # slow passthru
+        $r->send_http_header('text/xml');
+        eval {
+            my $fh = $provider->get_fh;
+            $r->send_fd($fh);
+        };
+        if ($@) {
+            my $str = $provider->get_strref;
+            $r->print($str);
+        }
+        return OK;
+    }
     
+    local $AxKit::Cache;
+
     my $retcode = eval {
         # $r->header_out('X-AxKit-Version', $VERSION);
-        
+
         chdir(File::Basename::dirname($r->filename));
-        
+
         $AxKit::OrigType = $r->content_type('changeme');
         
-        # get preferred stylesheet and media type
-        my ($preferred, $media) = get_style_and_media();
-        AxKit::Debug(2, "media: $media, preferred style: $preferred");
-        
-        # get cache object
-        my $cache = Apache::AxKit::Cache->new($r, $r->filename() . '.gzip' . ($ENV{PATH_INFO} || ''), $preferred, $media, $r->notes('axkit_cache_extra'));
-        
-        my $recreate; # regenerate from source (not cached)
-        
-        my $styles = get_styles($media, $preferred, $cache, $provider);
-        
-        {
-            local $^W;
-            if ($preferred && ($styles->[0]{title} ne $preferred)) {
-                # we selected a style that didn't exist.
-                # Make sure we default the cache file, otherwise
-                # we setup a potential DoS
-                AxKit::Debug(3, "resetting cache with no preferred style ($preferred ne $styles->[0]{title})");
-                $cache = Apache::AxKit::Cache->new($r, $r->filename() . '.gzip' . $ENV{PATH_INFO}, '', $media, $r->notes('axkit_cache_extra'));
-            }
-        }
-        
-        if (!$cache->exists()) {
-            AxKit::Debug(2, "cache doesn't exist");
-            # set no_cache header if cache doesn't exist due to no_cache option
-            $r->no_cache(1) if $cache->no_cache();
-            $recreate++;
-        }
-        
-        if (!$recreate && $AxKit::Cfg->DependencyChecks()) {
-            $recreate = check_dependencies($r, $provider, $cache);
-        }
-
-        if (!$recreate && $r->method() eq 'POST') {
-            $recreate++;
-        }
-
-        $AxKit::Charset = $AxKit::Cfg->OutputCharset();
-
-        if (!$recreate) {
-            AxKit::Debug(1, "delivering cached copy - all conditions met");
-            return $cache->deliver();
-            AxKit::Debug(1, "UNREACHABLE CODE!!!");
-        }
-
-        AxKit::Debug(1, "some condition failed. recreating output");
-
-        # Store in package variable for other modules
-        $AxKit::Cache = $cache;
-
-        # reconsecrate Apache request object (& STDOUT) into our own class
-        bless $r, 'AxKit::Apache';
-        tie *STDOUT, 'AxKit::Apache', $r;
-
-        if (my $charset = $AxKit::Cfg->OutputCharset) {
-            AxKit::Debug(5, "Different output charset: $charset");
-            if (!$r->notes('axkit_passthru_type')) {
-                $r->content_type("text/html; charset=$charset");
-            }
-        }
-        
-        my $uri = $r->uri();
-        my $path_info = $r->path_info();
-        $uri =~ s/\Q$path_info\E$//;
-        $r->uri($uri);
-
         reset_depends();
 
-        {
-            # copy styles because we blat the copy
-            my @copy = @$styles;
-            $AxKit::_CurrentStylesheets = \@copy;
-        } 
-        # Main grunt of the work done here...
-        my $result_code = process_request($r, $provider, $AxKit::_CurrentStylesheets);
-
-        save_dependencies($r, $cache);
-        
-#        if (my $dom = $r->pnotes('dom_tree')) {
-#            AxKit::Debug(4, "got a dom_tree back - outputting that to the cache");
-#            $r->notes('resetstring', 1);
-#            my $output = $dom->toString;
-#            $r->print($output);
-#        }
+        my $result_code = run_axkit_engine($r, $provider);
 
         # restore $r
         if (ref($r) eq 'AxKit::Apache') {
@@ -353,7 +302,22 @@ sub main_handler {
                     if $E->{reason};
         }
         AxKit::Debug(4, "[DECLINED] From: $E->{-file} : $E->{-line}");
-        return DECLINED;
+        
+        $r->send_http_header('text/xml');
+        eval {
+            my $fh = $provider->get_fh;
+            $r->send_fd($fh);
+        };
+        if ($@) {
+            eval {
+                my $str = $provider->get_strref;
+                $r->print($str);
+            };
+            if ($@) {
+                return DECLINED;
+            }
+        }
+        return OK;
     }
     elsif ($E->isa('Apache::AxKit::Exception::Error')) {
         $r->log->error("[AxKit] [Error] $E->{-text}");
@@ -392,6 +356,137 @@ sub main_handler {
     return DECLINED;
 }
 
+sub run_axkit_engine {
+    my ($r, $provider) = @_;
+    
+    # get preferred stylesheet and media type
+    my ($preferred, $media) = get_style_and_media();
+    AxKit::Debug(2, "media: $media, preferred style: $preferred");
+
+    # get cache object
+    my $cache = Apache::AxKit::Cache->new($r, $r->filename() . '.gzip' . ($r->path_info() || ''), $preferred, $media, $r->notes('axkit_cache_extra'));
+
+    my $recreate = 0; # regenerate from source (not cached)
+
+    my $styles = get_styles($media, $preferred, $cache, $provider);
+
+    {
+        local $^W;
+        if ($preferred && ($styles->[0]{title} ne $preferred)) {
+            # we selected a style that didn't exist.
+            # Make sure we default the cache file, otherwise
+            # we setup a potential DoS
+            AxKit::Debug(3, "resetting cache with no preferred style ($preferred ne $styles->[0]{title})");
+            $cache = Apache::AxKit::Cache->new($r, $r->filename() . '.gzip' . $r->path_info(), '', $media, $r->notes('axkit_cache_extra'));
+        }
+    }
+
+    if (!$cache->exists()) {
+        AxKit::Debug(2, "cache doesn't exist");
+        # set no_cache header if cache doesn't exist due to no_cache option
+        $r->no_cache(1) if $cache->no_cache();
+        $recreate++;
+    }
+
+    if (!$recreate && $AxKit::Cfg->DependencyChecks()) {
+        $recreate = check_dependencies($r, $provider, $cache);
+    }
+
+    if (!$recreate && $r->method() eq 'POST') {
+        $recreate++;
+    }
+
+    $AxKit::Charset = $AxKit::Cfg->OutputCharset();
+
+    if (!$recreate) {
+        AxKit::Debug(1, "delivering cached copy - all conditions met");
+        return $cache->deliver();
+    }
+
+    AxKit::Debug(1, "some condition failed. recreating output");
+
+    # Store in package variable for other modules
+    $AxKit::Cache = $cache;
+
+    # reconsecrate Apache request object (& STDOUT) into our own class
+    bless $r, 'AxKit::Apache';
+    tie *STDOUT, 'AxKit::Apache', $r;
+
+    if (my $charset = $AxKit::Cfg->OutputCharset) {
+        AxKit::Debug(5, "Different output charset: $charset");
+        if (!$r->notes('axkit_passthru_type')) {
+            $r->content_type("text/html; charset=$charset");
+        }
+    }
+    
+    # This is here so that lookup_uri() works on the real thing
+    # that we're requesting, not on the thing plus the PATH_INFO
+    my $uri = $r->uri();
+    my $path_info = $r->path_info();
+    $uri =~ s/\Q$path_info\E$//;
+    $r->uri($uri);
+    $ENV{PATH_INFO} = $path_info;
+
+    {
+        # copy styles because we blat the copy
+        my @copy = @$styles;
+        $AxKit::_CurrentStylesheets = \@copy;
+    }
+    
+    # Main grunt of the work done here...
+    my $return_code = process_request($r, $provider, $AxKit::_CurrentStylesheets);
+
+    save_dependencies($r, $cache);
+    
+    return $return_code;
+        
+}
+
+sub get_axkit_uri {
+    my ($uri) = @_;
+    
+    AxKit::Debug(3, "get_axkit_uri($uri)");
+    
+    my $apache = AxKit::Apache->request;
+    my $r;
+    if ($uri =~ /^axkit:\/(\/.*)$/) {
+        my $abs_uri = $1;
+        $r = $apache->lookup_uri($abs_uri);
+    }
+    elsif ($uri =~ /^axkit:(.*)$/) {
+        my $rel_uri = $1;
+        $r = $apache->lookup_uri($rel_uri);
+    }
+    else {
+        throw Apache::AxKit::Exception (-text => "get_axkit_uri for non-axkit URIs is not yet supported");
+    }
+    
+    local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($r);
+    local $AxKit::Cache;
+
+    my $provider = Apache::AxKit::Provider->new_content_provider($r);
+    
+    my $result_code = run_axkit_engine($r, $provider);
+    
+    if ($result_code == OK) {
+        # results now in $r->pnotes('xml_string') - probably...
+        # warning; missing caching logic here from deliver_to_browser.
+        if (not $r->pnotes('xml_string') and $r->pnotes('dom_tree')) {
+            return $r->pnotes('dom_tree')->toString;
+        }
+        else {
+            return $r->pnotes('xml_string');
+        }
+    }
+    elsif ($result_code == DECLINED) {
+        # probably came from the cache system. Try and read it.
+        return $AxKit::Cache->read();
+    }
+    else {
+        throw Apache::AxKit::Exception ( -text => "$uri internal request returned unknown result code: ".$result_code);
+    }
+}
+
 sub process_error {
     my ($r, $E, $error_styles) = @_;
     
@@ -484,16 +579,26 @@ sub reset_stylesheets {
 sub process_request {
     my ($r, $provider, $styles) = @_;
     my $result_code = OK;
-    
+
     my $num_styles = 0;
     for my $style (@$styles) {
         AxKit::Debug(4, "styles: ", $style->{module}, "(", $style->{href}, ")");
         $num_styles++;
     }
 
+    my $interm_prefix;
+    my $interm_count = 0;
+    if ($AxKit::Cfg->TraceIntermediate) {
+        $interm_prefix = $r->uri;
+        $interm_prefix =~ s{/}{|}g;
+        $interm_prefix =~ s/[^0-9a-zA-Z.,_|-]/_/g;
+        $interm_prefix = $AxKit::Cfg->TraceIntermediate.'/'.$interm_prefix;
+    }
+
     while (@$styles) {
         my $style = shift @$styles;
-        my $styleprovider = Apache::AxKit::Provider->new(
+
+        my $styleprovider = Apache::AxKit::Provider->new_style_provider(
                 $r,
                 uri => $style->{href},
                 );
@@ -505,7 +610,7 @@ sub process_request {
         my $mapto = $style->{module};
 
         AxKit::load_module($mapto);
-        
+
         AxKit::Debug(3, "about to execute: $mapto\::handler");
 
         my $method = "handler";
@@ -516,12 +621,12 @@ sub process_request {
                         );
             }
             my $retval = $mapto->$method(
-                    $r, 
-                    $provider, 
-                    $styleprovider, 
+                    $r,
+                    $provider,
+                    $styleprovider,
                     !@$styles, # any more left?
                     );
-	        $result_code = $retval if $retval != OK;
+            $result_code = $retval if $retval != OK;
         }
         else {
             throw Apache::AxKit::Exception::Error(
@@ -529,6 +634,23 @@ sub process_request {
                 );
         }
 
+        if ($interm_prefix) {
+            my $fh = Apache->gensym();
+            if (sysopen($fh, $interm_prefix.'.'.$interm_count, O_WRONLY|O_CREAT|O_TRUNC)) {
+                if (my $dom_tree = $r->pnotes('dom_tree')) {
+                    syswrite($fh,$dom_tree->toString);
+                } elsif (my $xmlstr = $r->pnotes('xml_string')) {
+                    syswrite($fh,$xmlstr);
+                } else {
+                    syswrite($fh,"<?xml version='1.0'?>\n<empty reason='no data found'/>");
+                }
+                close($fh);
+	        $interm_count++;
+            } else {
+                AxKit::Debug(1,"could not open $interm_prefix.$interm_count for writing: $!");
+            }
+        }
+
         AxKit::Debug(3, "execution of: $mapto\::$method finished");
 
         last if $r->notes('axkit_passthru');
@@ -610,9 +732,9 @@ sub check_dependencies {
 
 sub save_dependencies {
     my ($r, $cache) = @_;
-    
+
     return if $cache->no_cache();
-    
+
     eval {
         my @depends = get_depends();
         my $depend_cache = Apache::AxKit::Cache->new($r, $cache->key(), '.depends');
@@ -630,7 +752,7 @@ sub deliver_to_browser {
     if (not $r->pnotes('xml_string') and $r->pnotes('dom_tree')) {
         $r->pnotes('xml_string', $r->pnotes('dom_tree')->toString );
     }
-    
+
     if ($r->content_type eq 'changeme' && !$r->notes('axkit_passthru_type')) {
         $AxKit::Cfg->AllowOutputCharset(1);
         $r->content_type('text/html; charset=' . ($AxKit::Cfg->OutputCharset || "UTF-8"));
@@ -638,25 +760,25 @@ sub deliver_to_browser {
     elsif ($r->notes('axkit_passthru_type')) {
         $r->content_type($AxKit::OrigType);
     }
-    
+
     if (my $charset = $AxKit::Cfg->OutputCharset()) {
         my $ct = $r->content_type;
         $ct =~ s/charset=.*?(;|$)/charset=$charset/i;
         $r->content_type($ct);
     }
-            
-    if ($result_code != OK) {
+
+    if ($result_code != OK && $result_code != 200) {
     	# no caching - probably makes no sense, and will be turned off
-	    # anyways, as currently only XSP pages allow to send custom responses
+        # anyways, as currently only XSP pages allow to send custom responses
     	AxKit::Debug(4,"sending custom response: $result_code");
         my ($transformer, $doit) = get_output_transformer();
         if ($doit) {
-        	$r->custom_response($result_code,$transformer->($r->pnotes('xml_string')));
+            $r->custom_response($result_code,$transformer->($r->pnotes('xml_string') || ''));
         }
-         else {
-         	$r->custom_response($result_code,$r->pnotes('xml_string'));
-	    }
-	    return $result_code;
+        else {
+            $r->custom_response($result_code,$r->pnotes('xml_string') || '');
+        }
+        return $result_code;
     }
 
     if ($AxKit::Cache->no_cache() ||
@@ -668,12 +790,12 @@ sub deliver_to_browser {
             AxKit::Debug(4, 'Sending gzipped xml string to browser');
             AxKit::Apache::send_http_header($r);
             if ($doit) {
-                $r->print( Compress::Zlib::memGzip(
+                $r->print( unpack("U0A*", Compress::Zlib::memGzip( 
                          $transformer->( $r->pnotes('xml_string') )
-                         ) );
+                         ) ) );
             }
             else {
-                $r->print( Compress::Zlib::memGzip( $r->pnotes('xml_string') ) );
+                $r->print( unpack("U0A*", Compress::Zlib::memGzip( $r->pnotes('xml_string') ) ) );
             }
         }
         else {
@@ -817,7 +939,7 @@ sub print {
         $self->pnotes('xml_string', '');
         $self->notes('resetstring', 0);
     }
-    
+
     my $current = $self->pnotes('xml_string');
     $self->pnotes('xml_string', $current . join('', @_));
 }
@@ -954,6 +1076,16 @@ use this option on a live server.
 
     AxDebugLevel 5
 
+=head2 AxTraceIntermediate
+
+With this option you advise AxKit to store the result of each transformation
+request in a special directory for debugging. This directory must exist and must
+be writeable by the httpd. The files are stored with their full uri, replacing
+slashes with '|', and appending a number indicating the transformation step.
+'.0' is the xml after the first transformation.
+
+    AxTraceIntermediate /tmp/axkit-trace
+
 =head2 AxStackTrace
 
 This FLAG option says whether to maintain a stack trace with every exception.
@@ -1098,6 +1230,21 @@ If you prefix the module name with a + sign, it will be pre-loaded on
 server startup (assuming that the config directive is in a httpd.conf,
 rather than a .htaccess file).
 
+=head2 AxIgnoreStylePI
+
+Turn off parsing and overriding stylesheet selection for XML files containing
+an C<xml-stylesheet> processing instruction at the start of the file. This is
+a FLAG option - On or Off. The default value is "Off".
+
+  AxIgnoreStylePI On
+
+=head2 AxHandleDirs
+
+Enable this option to allow AxKit to process directories. Uses XML::Directory
+and XML::SAX::Writer to create the directory listing.
+
+  AxHandleDirs On
+
 =head2 AxStyle
 
 A default stylesheet title to use. This is useful when a single XML
@@ -1145,8 +1292,10 @@ a directory processed by a different XSLT engine.
 
 There are several directives specifically designed to allow you to build
 a flexible sitemap that specifies how XML files get processed on your
-system. These directives are used only if your XML file does not have the
-<?xml-stylesheet?> directives.
+system.
+
+B<Note:> <?xml-stylesheet?> directives in your XML files override these
+directives unless you enable the AxIgnoreStylePI option listed above.
 
 =head2 AxAddProcessor
 
@@ -1268,15 +1417,18 @@ These directives all take as a single argument, the name of a module
 to load in place of the default. They are:
 
     AxConfigReader
-    AxProvider
+    AxContentProvider
+    AxStyleProvider
     AxCacheModule
 
 The ConfigReader module returns information about various configuration
 options. Currently it takes most of its information from the above
 mentioned configuration directives, or from PerlSetVar.
 
-The Provider module is the means by which AxKit gets its resources from.
-The default Provider simply picks up files from the filesystem, but
+The Provider modules are the means by which AxKit gets its resources from.
+ContentProviders deliver up the document to be processed, while StyleProviders
+are used to get the data for any stylesheets that will be applied.
+The default Provider for each simply picks up files from the filesystem, but
 alternate providers could pull the information from a DBMS, or perhaps
 create some XML structure for directories. There currently exists one
 alternate Provider module, which allows AxKit to work as a recipient
